Library "setupCommon.brs"

Sub Main()

	version="1.0"
	print "setupChannels.brs version ";version;" started"

	CheckModelType()

	CheckFirmwareVersion()

	CheckStorageDeviceIsWritable()

	RunScanner()

End Sub


Sub DisplayMessage(msg1$ As String, msg2$ As String, msg3$ As String)

	videoMode = CreateObject("roVideoMode")
	resX = videoMode.GetResX()
	resY = videoMode.GetResY()
	videoMode = invalid

	twParams = CreateObject("roAssociativeArray")
	twParams.LineCount = 1
	twParams.TextMode = 2
	twParams.Rotation = 0
	twParams.Alignment = 1

	r = CreateObject("roRectangle", 0, 0, resX, resY)
	tw=CreateObject("roTextWidget",r,1,2,twParams)
	tw.PushString("")
	tw.Show()

	r=CreateObject("roRectangle",0,resY/2-resY/32,resX,resY/32)
	tw=CreateObject("roTextWidget",r,1,2,twParams)
	tw.PushString(msg1$)
	tw.Show()

	r2=CreateObject("roRectangle",0,resY/2,resX,resY/32)
	tw2=CreateObject("roTextWidget",r2,1,2,twParams)
	tw2.PushString(msg2$)
	tw2.Show()

	r3=CreateObject("roRectangle",0,resY/2+resY/32,resX,resY/32)
	tw3=CreateObject("roTextWidget",r3,1,2,twParams)
	tw3.PushString(msg3$)
	tw3.Show()

	msgPort = CreateObject("roMessagePort")
	gpioPort = CreateObject("roGpioControlPort")
	gpioPort.SetPort(msgPort)
	while true
		msg = wait(0, msgPort)
		if type(msg) = "roGpioButton" and msg.GetInt()=12 then
			stop
		endif
	end while
End Sub


Sub CheckModelType()

	modelObject = CreateObject("roDeviceInfo")
	if modelObject.GetModel() <> "XD1230" then
		DisplayMessage("This device does not support tuning.", "", "")
	endif

End Sub


Sub RunScanner() As Integer

	Scanner = newScanner()

	Scanner.ParseSetup()

	Scanner.PerformScan()

End Sub


Function newScanner() As Object

	Scanner = { }

	Scanner.ParseSetup = ParseSetup
	Scanner.PerformScan = PerformScan

	return scanner

End Function


Sub ParseSetup()

	m.channelMap = invalid
	m.modulationType = invalid
	m.firstRFChannel = invalid
	m.lastRFChannel = invalid

    ' retrieve settings from XML file
	if len(ReadAsciiFile("setupChannels.xml")) = 0 then
		DisplayMessage("The file setupChannels.xml is missing.", "", "")
	endif

    xml = CreateObject("roXMLElement")
    xml.Parse(ReadAsciiFile("setupChannels.xml"))

	m.channelMap = GetNamedElement(xml, "channelMap")
	m.modulationType = GetNamedElement(xml, "modulationType")
	m.firstRFChannel = GetNamedElement(xml, "firstRFChannel")
	m.lastRFChannel = GetNamedElement(xml, "lastRFChannel")

End Sub


Sub PerformScan()

	' setup screen to display messages
	videoMode = CreateObject("roVideoMode")
	resX = videoMode.GetResX()
	resY = videoMode.GetResY()
	videoMode = invalid
	r=CreateObject("roRectangle",0,resY/2-resY/64,resX,resY/32)
	twParams = CreateObject("roAssociativeArray")
	twParams.LineCount = 1
	twParams.TextMode = 2
	twParams.Rotation = 0
	twParams.Alignment = 1
	tw=CreateObject("roTextWidget",r,1,2,twParams)
	tw.Show()

	msgPort = CreateObject("roMessagePort")

	channelManager = CreateObject("roChannelManager")
	channelManager.SetPort(msgPort)

	' Fill in parameters to limit the scan
	scanSpec  = CreateObject("roAssociativeArray")

	if type(m.channelMap) = "roString" and len(m.channelMap) > 0 and m.channelMap <> "ATSC_QAM" then
		scanSpec["ChannelMap"] = m.channelMap
	endif

	if not(type(m.channelMap) = "roString" and m.channelMap = "ATSC") then
		if type(m.modulationType) = "roString" and len(m.modulationType) > 0 and m.modulationType <> "QAM64_QAM256" then
			scanSpec["ModulationType"] = m.modulationType
		endif
	endif

	if type(m.firstRFChannel) = "roString" and len(m.firstRFChannel) > 0 then
		firstRFChannel% = int(val(m.firstRFChannel))
		scanSpec["FirstRfChannel"] = firstRFChannel%
	endif

	if type(m.lastRFChannel) = "roString" and len(m.lastRFChannel) > 0 then
		lastRFChannel% = int(val(m.lastRFChannel))
		scanSpec["LastRfChannel"] = lastRFChannel%
	endif

	' Scan the channels
	channelManager.ClearChannelData()
	channelManager.AsyncScan(scanSpec)

	percentageComplete% = 0
	numChannelsFound% = 0

	percentageCompleteMsg$ = "Scanning 0% complete."
	numChannelsFoundMsg$ = " Found 0 channels."

	while true

		event = wait(0, msgPort)

		if type(event) = "roChannelManagerEvent" then

			if event = 0 then
			
				print "Scan complete."

				exit while
			
			else if event = 1 then

				percentageComplete% = event.GetData()

				print "scan percentage complete = ";percentageComplete%

				channelDescriptor = event.GetChannelDescriptor()

				print "Scanning RF channel ";channelDescriptor.RfChannel

				percentageCompleteMsg$ = "Scanning" + stri(percentageComplete%) + "% complete."

				tw.PushString(percentageCompleteMsg$ + numChannelsFoundMsg$)

			else if event = 2 then

				channelDescriptor = event.GetChannelDescriptor()

				print "Found channel ";channelDescriptor.VirtualChannel
				
				numChannelsFound% = numChannelsFound% + 1

				numChannelsFoundMsg$ = " Found " + stri(numChannelsFound%) + " channels."

				tw.PushString(percentageCompleteMsg$ + numChannelsFoundMsg$)

			endif

		endif

	end while

	' Generate the data that BrightAuthor / autorun might use
	channelCount% = channelManager.GetChannelCount()
	channelDescriptors = CreateObject("roArray", channelCount%, false)

	if channelCount% > 0 then
		channelInfo  = CreateObject("roAssociativeArray")
		for channelIndex% = 0 to channelCount% - 1
			channelInfo["ChannelIndex"] = channelIndex%
			channelDescriptor = channelManager.CreateChannelDescriptor(channelInfo)
			channelDescriptors.push(channelDescriptor)
		next
	endif

' Generate XML data for the results of the scan
	docName$ = "BrightSignRFChannels"

    root = CreateObject("roXMLElement")
    root.SetName(docName$)
	root.AddAttribute("version", "1.0")

	rfChannels = root.AddElement("rfChannels")

	for each channelDescriptor in channelDescriptors

		channelDescriptorElement		= rfChannels.AddElement("rfInputChannel")
		virtualChannelElement			= channelDescriptorElement.AddElement("virtualChannel")
		channelNameElement				= channelDescriptorElement.AddElement("channelName")

	    virtualChannelElement.SetBody(channelDescriptor.VirtualChannel)
	    channelNameElement.SetBody(channelDescriptor.ChannelName)
	next

    xml = root.GenXML({ indent: " ", newline: chr(10), header: true })

' Retrieve the XML data for the tuning data produced by system software
	scannedChannelData$ = channelManager.ExportToXml()

' Embed the scanned channel data in the xml as a CDATA element

	' find the insert point in the xml (is there a way to write out a proper element?)
	index% = instr(1, xml, "</" + docName$ + ">")
	if index% > 0 then
		xml = mid(xml, 1, index% - 1)
		xml = xml + " <![CDATA[" + chr(10) + scannedChannelData$ + " ]]>" + chr(10) + "</" + docName$ + ">"
	endif

	ok = WriteAsciiFile("ScannedChannels.xml", xml)

' Indicate completion
	msg1$ = "Scan complete - " + StripLeadingSpaces(stri(numChannelsFound%)) + " channels found."
	if not ok then
		msg2$ = "Error while writing the scanned file to the card."
		msg3$ = ""
	else
		msg2$ = "The file ScannedChannels.xml contains the scanned channel data and can be imported into BrightAuthor."
		msg3$ = "You may now remove the card."
	endif

	DisplayMessage(msg1$, msg2$, msg3$)

End Sub


Function GetNamedElement(xml As Object, elementName$ As String) As String

    ele = xml.GetNamedElements(elementName$)
    if type(ele) <> "roXMLList" then
        return ""
    endif
    if ele.Count() <> 1 then
        return ""
    endif
    
    return ele.GetText()    
    
End Function


Function StripLeadingSpaces(inputString$ As String) As String

    while true
        if left(inputString$, 1)<>" " then return inputString$
        inputString$ = right(inputString$, len(inputString$)-1)
    endwhile

    return inputString$

End Function
